#ifndef	__SPI1_DRIVER_h__
#define	__SPI1_DRIVER_h__
//==================================================================================
// The information contained herein is the exclusive property of
// Generalplus Technology Co. And shall not be distributed, reproduced,
// or disclosed in whole in part without prior written permission.
//       (C) COPYRIGHT 2009   Generalplus TECHNOLOGY CO.                            
//                   ALL RIGHTS RESERVED
// The entire notice above must be reproduced on all authorized copies.
//==================================================================================
//==================================================================================
// Name                  : SPI1_Driver.inc
// Applied Body          : GPL951
// Programmer            : 
// Description           : SPI1 driver header file
// History version       : 
// Rev #      Date         Who			Comments
// -----	 -----------   ---------	--------------------------------------------
// 1.0	     2016/04/14   JackyLin		Oringinal Version
// 1.1		 2017/04/12   JackyLin		1. _SPI0_WriteNByte function bug fix.
//										2. Add slave mode transmission function code and MasterMode/SlaveMode define
//==================================================================================

//==========================================
// Include file area
//==========================================


//==========================================
// Constant define area
//==========================================

//==========================================
// External declare area
//==========================================
extern int R_SPI1_ReadOneByteData;
extern int Slave_SPI1_ReadOneByteData;

// ======================================================================================
// Function name: SPI1_Intial
// Purpose		: SPI1 interface initial 
// Parameter	: SPI_CLK_SEL. SPI clcok is system clock divided by 2/4/8/16/32/64/128 
//				  Here, SPI_CLK_SEL=0 is system clock/2. SPI_CLK_SEL=1 is system clock/4. ... SPI_CLK_SEL=7 is system clock/128.  
// Return		: None
// ======================================================================================
extern void SPI1_Intial(int SPI_CLK_SEL);

// ======================================================================================
// Function name: SPI1_CSHigh
// Purpose		: SPI1 chip selection set to high 
// Parameter	: None 
// Return		: None
// ======================================================================================
extern void	SPI1_CSHigh(void);


// ======================================================================================
// Function name: SPI1_CSLow
// Purpose		: SPI1 chip selection set to low 
// Parameter	: None 
// Return		: None
// ======================================================================================
extern void	SPI1_CSLow(void);


// ======================================================================================
// Function name: SPI0_Close
// Purpose		: SPI0 function Disable.  
// Parameter	: None 
// Return		: None 
// ======================================================================================
extern void	SPI1_Close(void);


// ======================================================================================
// Function name: SPI1_WriteOneByte
// Purpose		: SPI transmit one byte data 
// Parameter	: int TransmitData: transmit data 
// Return		: 0: complete
//				  -1: Timeout
//				  read back data will put in R_SPI1_ReadOneByteData variable 		
// ======================================================================================
extern int SPI1_WriteOneByte(int TransmitData);


// ======================================================================================
// Function name: SPI1_WriteNByte
// Purpose		: SPI N byte write(from buffer) 
// Parameter	: int* TXDataPoint: transmit data address
//				  int DataLength: transmit data length  
// Return		: 0: complete
//				  -1: Timeout
// ======================================================================================
extern int SPI1_WriteNByte(int* TXDataPoint, int DataLength);


// ======================================================================================
// Function name: SPI1_ReadNByte
// Purpose		: SPI N byte write(from buffer) 
// Parameter	: int* RXDataPoint: store receive data point
//				  int DataLength: receive data length 
// Return		: 0: complete
//				  -1: Timeout
// ======================================================================================	
extern int SPI1_ReadNByte(int* RXDataPoint, int dataLength);


// ======================================================================================
// Function name: SPI1_TransmissionDMA
// Purpose		: SPI1 transmission by DMA. 
// Parameter	: int* R_SPI1_TXDataPoint: transmit data buffer Address. 
//				  int* R_SPI1_RXDataPoint: receive data buffer Address.
//				  long R_SPI1_DataLength: transmission data length
//				  int Mode: transmission mode. here, Mode=0,transmit address is constant and transmit address is byte mode, receive address is constant and recevie address is byte mode.   
//				    		  						 Mode=1,transmit address is increase and transmit address is byte mode, receive address is constant and recevie address is byte mode.	
//				    		  						 Mode=2,transmit address is constant and transmit address is word mode, receive address is constant and recevie address is byte mode.	
//				    		  						 Mode=3,transmit address is increase and transmit address is word mode, receive address is constant and recevie address is byte mode.	
//				    		  						 Mode=4,transmit address is constant and transmit address is byte mode, receive address is increase and recevie address is byte mode.	
//				    		  						 Mode=5,transmit address is increase and transmit address is byte mode, receive address is increase and recevie address is byte mode.	
//				    		  						 Mode=6,transmit address is constant and transmit address is word mode, receive address is increase and recevie address is byte mode.	
//				    		  						 Mode=7,transmit address is increase and transmit address is word mode, receive address is increase and recevie address is byte mode.	
//				    		  						 Mode=8,transmit address is constant and transmit address is byte mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=9,transmit address is increase and transmit address is byte mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=10,transmit address is constant and transmit address is word mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=11,transmit address is increase and transmit address is word mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=12,transmit address is constant and transmit address is byte mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=13,transmit address is increase and transmit address is byte mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=14,transmit address is constant and transmit address is word mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=15,transmit address is increase and transmit address is word mode, receive address is constant and recevie address is word mode.	
// Return		: 0: complete
//				  -1: Timeout
// ======================================================================================
extern int SPI1_TransmissionDMA(int* R_SPI1_TXDataPoint, int* R_SPI1_RXDataPoint, long R_SPI1_DataLength, int Mode);



// ======================================================================================
// Function name: Slave_SPI1_Intial
// Purpose		: SPI1 interface slave mode initial 
// Parameter	: None  
// Return		: None
// ======================================================================================
extern void Slave_SPI1_Intial(void);


// ======================================================================================
// Function name: Slave_SPI1_WriteOneByte
// Purpose		: SPI slave mode transmit and receive one byte data.               
// Parameter	: int TransmitData: transmit data 
// Return		: None.
//				  After this function call. User need call Slave_SPI1_Status to polling is master read data out?  
//                If Slave_SPI1_Status return 0, the read data will be put in Slave_SPI1_ReadOneByteData
// ======================================================================================	
extern void Slave_SPI1_WriteOneByte(int TransmitData);


// ======================================================================================
// Function name: Slave_SPI1_WriteNByte
// Purpose		: slave mode SPI N byte write(from buffer). here, the read data will be give up. 
// Parameter	: int* TXDataPoint: TX data buffer Address, 
//				  int DataLength: transmit data length  
// Return		: None
//				  After this function call. User need call Slave_SPI1_Status to polling is master read data out?
//                If Slave_SPI1_Status return 0. Means transmit data is finish.  
// ======================================================================================	
extern void Slave_SPI1_WriteNByte(int* TXDataPoint, int DataLength);


// ======================================================================================
// Function name: Slave_SPI1_ReadNByte
// Purpose		: Slave mode SPI N byte Read(to buffer). Here, SPI will send zero data out.  
// Parameter	: int* RXDataPoint: receive data buffer Address.
//				  int R_SPI1_DataLength: receive data length 
// Return		: None
//				  After this function call. User need call Slave_SPI1_Status to polling is master send data out?  
//                If Slave_SPI1_Status return 0. Means receive data is finish.
// ======================================================================================	
extern void Slave_SPI1_ReadNByte(int* TXDataPoint, int DataLength);


// ======================================================================================
// Function name: Slave_SPI1_ISRService
// Purpose		: SPI1 slave mode interrupt service code. Put this function to SPI1 RX interrupt service routine   
// Parameter	: None 
// Return		: None
// ======================================================================================
extern void Slave_SPI1_ISRService(void);


// ======================================================================================
// Function name: Slave_SPI1_Status
// Purpose		: Slave mode data tramssion status check. After slave mode data transmit/receive function called. user need call this function to check is master had to teansmit/receive data.   
// Parameter	: None 
// Return		: remain data length.
//				  if return 0, means data transsiom is finish.
//				  if return 0xFFFF, means SPI slave receive over run. There are data mission.		
// ======================================================================================
extern int Slave_SPI1_Status(void);

#endif
