//==================================================================================
// The information contained herein is the exclusive property of
// Generalplus Technology Co. And shall not be distributed, reproduced,
// or disclosed in whole in part without prior written permission.
//       (C) COPYRIGHT 2009   Generalplus TECHNOLOGY CO.                            
//                   ALL RIGHTS RESERVED
// The entire notice above must be reproduced on all authorized copies.
//==================================================================================
//==================================================================================
// Name                  : SPI_DriverCode.asm
// Applied Body          : GPL951
// Programmer            : 
// Description           : SPI driver code
// History version       : 
// Rev #      Date         Who			Comments
// -----	 -----------   ---------	--------------------------------------------
// 1.0	     2016/04/14   JackyLin		Oringinal Version
// 1.1		 2017/04/12   JackyLin		1. _SPI0_WriteNByte function bug fix.
//										2. Add slave mode transmission function code and MasterMode/SlaveMode define
//==================================================================================

//==========================================
// Include file area
//==========================================
.INCLUDE	GPL951_Body.inc

//==========================================
// Constant define area
//==========================================
.define	C_SPI_CLKPhase0		0x0000	
.define	C_SPI_CLKPhase1		0x0020
.define	C_SPI_CLKPolarity0	0x0000
.define	C_SPI_CLKPolarity1	0x0010


//----- user define decision-----
.define	Addressmode		1			//1:word mode, 1 address store 2 bytes. 
									//0:byte mode, 1 address atore 1 byte.    

.define	MasterMode		1			//1:support SPI master mode code. 0: not support SPI master mode.
									//user set it to 0 for save code szie. 

.define	SlaveMode		1			//1: support SPI slave mode code. 0: not support SPI slave mode		
				                    //user set it to 0 for save code szie. 									
									
.define	SPI1_CLKPhase		C_SPI_CLKPhase0			//in slave mode, C_SPI_CLKPhase0 supported only.
.define	SPI1_CLKPolarity	C_SPI_CLKPolarity0		//in slave mode, C_SPI_CLKPolarity0 supported only.
.define	SPI1_CSPort			P_IOF_Data	
.define	SPI1_CSpin			0x02



//==========================================
// External declare area
//==========================================
.external F_SYS_GetCount

.if MasterMode
//==========================================
// Public declare area
//==========================================
.public _SPI1_Intial, F_SPI1_Intial
.public _SPI1_CSHigh, F_SPI1_CSHigh
.public _SPI1_CSLow, F_SPI1_CSLow
.public _SPI1_WriteOneByte, F_SPI1_WriteOneByte
.public _SPI1_WriteNByte, F_SPI1_WriteNByte
.public _SPI1_ReadNByte, F_SPI1_ReadNByte
.public _SPI1_TransmissionDMA, F_SPI1_TransmissionDMA
.public _SPI1_Close, F_SPI1_Close


.public	_SPI1_ReadOneByteData, R_SPI1_ReadOneByteData
.public	R_SPI1_TXDataPoint
.public	R_SPI1_RXDataPoint
.public R_SPI1_DataLength
.public R_SPI1_Mode

//==========================================
//Variable RAM declare area
//==========================================
SPI1_RAM:	.section	.ram	
.var	R_SPI1_ReadOneByteData
.define _SPI1_ReadOneByteData	R_SPI1_ReadOneByteData
.define	R_SPI1_Mode				R_SPI1_ReadOneByteData
R_SPI1_TXDataPoint: .dw 2 DUP(?)
R_SPI1_RXDataPoint: .dw 2 DUP(?)	
R_SPI1_DataLength:	.dw	2 DUP(?)

//==========================================
// code starting 
//==========================================
SPI1_DriverCode: .section	.code
// ======================================================================================
// Function name: SPI1_Intial
// Purpose		: SPI1 interface initial 
// Parameter	: int SPI_CLK_SEL. SPI clcok is system clock divided by 2/4/8/16/32/64/128
//				  Here, SPI_CLK_SEL=0 is system clock/2. SPI_CLK_SEL=1 is system clock/4. ... SPI_CLK_SEL=7 is system clock/128.  
// Return		: None
// Destroy		: r1,r2 
// ======================================================================================
_SPI1_Intial:	.proc
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]			//SPI_CLK_SEL
	call F_SPI1_Intial
	pop bp from [sp]
	retf

F_SPI1_Intial:
	r2 =0x0800			//reset
	[P_SPI1_Ctrl]=r2
	
	r2=[SPI1_CSPort+3]		//set SPI cs output high 
	r2|=SPI1_CSpin
	[SPI1_CSPort+3]=r2	
	r2=[SPI1_CSPort+2]
	r2|=SPI1_CSpin
	[SPI1_CSPort+2]=r2
	r2=[SPI1_CSPort]
	r2|=SPI1_CSpin
	[SPI1_CSPort]=r2
	
	r1&=0x07
	r1|=(C_SPI1_Enable+SPI1_CLKPhase+SPI1_CLKPolarity)
	[P_SPI1_Ctrl]=r1
		
	r1=[P_SPI1_Misc]		//enable smart mode
	r1|=C_SPI1_FIFO_Smart_Mode
	[P_SPI1_Misc]=r1
	retf	
	.endp
	


// ======================================================================================
// Function name: F_SPI1_CSHigh
// Purpose		: SPI1 chip selection set to high 
// Parameter	: None 
// Return		: None
// Destroy		: r1 
// ======================================================================================
F_SPI1_CSHigh:
_SPI1_CSHigh:
	r1=[SPI1_CSPort+1]		//set SPI cs output high 
	r1|=SPI1_CSpin
	[SPI1_CSPort+1]=r1	
	retf
	

// ======================================================================================
// Function name: F_SPI1_CSLow
// Purpose		: SPI1 chip selection set to low 
// Parameter	: None 
// Return		: None
// Destroy		: r1 
// ======================================================================================
F_SPI1_CSLow:
_SPI1_CSLow:
	r1=[SPI1_CSPort+1]		//set SPI cs output low 
	r1&=~SPI1_CSpin
	[SPI1_CSPort+1]=r1	
	retf


// ======================================================================================
// Function name: F_SPI1_Close
// Purpose		: SPI1 function Disable.  
// Parameter	: None 
// Return		: None
// Destroy		: r1 
// ======================================================================================
_SPI1_Close:
F_SPI1_Close:
	r1=0x0800
	[P_SPI1_Ctrl]=r1
	retf

// ======================================================================================
// Function name: SPI1_WriteOneByte
// Purpose		: SPI transmit one byte data 
// Parameter	: int TXData 
// Return		: r1=0: complete
//				  r1=-1: Timeout
//				  read back data will put in SPI_ReadOneByteData(R_SPI_ReadOneByteData) variable 		
// Destroy		: r1,r2 
// ======================================================================================	
_SPI1_WriteOneByte:		
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]			//TX Data
	call F_SPI1_WriteOneByte
	pop bp from [sp]
	retf

F_SPI1_WriteOneByte:
	[P_SPI1_TXData]=r1	
	call F_SYS_GetCount	
	r2=r1+2
L_WaitWriteOneByteIdle?:
	call F_SYS_GetCount	
	cmp	r1,r2
	je	L_SPIWriteOneByteTimeout?   
RX_Ready?:
	r1=[P_SPI1_RXStatus]
	r1&=0x07
	jz	L_WaitWriteOneByteIdle?
	r1=[P_SPI1_RXData]
	[R_SPI1_ReadOneByteData]=r1	
	r1=0
	retf	

L_SPIWriteOneByteTimeout?:	
    r1=-1
    retf	 


// ======================================================================================
// Function name: SPI1_WriteNByte
// Purpose		: SPI N byte write(from buffer) 
// Parameter	: int* R_SPI1_TXDataPoint: TX data buffer Address, 
//				  int DataLength: transmit data length  
// Return		: r1=0: complete
//				  r1=-1: Timeout
// Destroy		: r1,r2,r3,r4,r5 
// ======================================================================================	
_SPI1_WriteNByte:
	push	bp to [sp]
	bp=sp+1
	r4=[bp+3]					//transmit data point low
	r3=[bp+4]					//transmit data point high	
	r5=[bp+5]					//data length
	[R_SPI1_DataLength]=r1
	call F_SPI1_WriteNByte
	pop bp from [sp]
	retf

F_SPI1_WriteNByte:
	cmp	r5,0 					//if length 0 then end TX
	je	endTX?
	ds=r3						//data point ds
L_TransLoop?:
	cmp	r5,8
	jb	L_SPIWriteShortFIFO?
L_SPISequenceWrite?:			//length large than 8, then transmit 8 byte once.		
	r2=8
writeloop?:	
	r3=ds:[r4++]
	[P_SPI1_TXData]=r3	//low byte first
.if Addressmode
	r3=r3 lsr 4
	r3=r3 lsr 4
	[P_SPI1_TXData]=r3
	r2-=2
.else
	r2-=1
.endif		
	jnz	writeloop?
	
	call F_SYS_GetCount		
	r2=r1+3
waitrx?:					//detect 8 byte tx finish?
	call F_SYS_GetCount	
	cmp	r1,r2
	je	L_SPITimeout?
	r3=[P_SPI1_Misc]
	test r3,C_SPI1_RX_Full_Flag
	jz waitrx?		
	r2=8					//if tx finish, then read out RX FIFO data
cleRxFIFO?:
	r1=[P_SPI1_RXData]	
	r2-=1
	jnz	cleRxFIFO?
	r5-=8
	jnz	L_TransLoop?	
endTX?:
	r1=0
	retf
	
L_SPIWriteShortFIFO?:
	r2=r5	
writeloop1?:	
	r3=ds:[r4++]
	[P_SPI1_TXData]=r3	//low byte first
.if Addressmode	
	r2-=1
	jz txfinish?		
	r3=r3 lsr 4
	r3=r3 lsr 4
	[P_SPI1_TXData]=r3
	r2-=1
.else
	r2-=1
.endif	
	jnz	writeloop1?

txfinish?:
	call F_SYS_GetCount		
	r2=r1+3
waitrx1?:					//detect remain byte tx finish?
	call F_SYS_GetCount	
	cmp	r1,r2
	je	L_SPITimeout?
	r3=[P_SPI1_RXStatus]
	r3&=0x07
	cmp	r3,r5
	jne waitrx1?			
	r2=r5					//if tx finish, then read out RX FIFO data
cleRxFIFO1?:
	r1=[P_SPI1_RXData]	
	r2-=1
	jnz	cleRxFIFO1?
	r1=0
	retf

L_SPITimeout?:
	r1=-1
	retf

	

// ======================================================================================
// Function name: SPI1_ReadNByte
// Purpose		: SPI N byte write(from buffer) 
// Parameter	: int* R_SPI1_RXDataPoint: receive data buffer Address.
//				  int R_SPI1_DataLength: receive data length 
// Return		: r1=0: complete
//				  r1=-1: Timeout
// Destroy		: r1,r2,r3,r4,r5 
// ======================================================================================	
_SPI1_ReadNByte:
	push	bp to [sp]
	bp=sp+1
	r4=[bp+3]					//receive data point low 	
	r3=[bp+4]					//receive data point high
	r5=[bp+5]					//data length
	[R_SPI1_DataLength]=r1
	call F_SPI1_ReadNByte
	pop bp from [sp]
	retf

SPI1_EndRX:
	r1=0
	retf

F_SPI1_ReadNByte:
	cmp r5,0				//if data length 0 then RX end
	je	SPI1_EndRX
	ds=r3					//receive data point ds
L_TransLoop?:
	cmp	r5,8
	jb	L_SPIWriteShortFIFO?
L_SPISequenceWrite?:			//length large than 8, then transmit 8 byte once.		
	r2=8
writeloop?:	
	r3=0						//send dummy byte
	[P_SPI1_TXData]=r3	
	r2-=1
	jnz	writeloop?
	
	call F_SYS_GetCount		
	r2=r1+3
waitrx?:					//detect 8 byte tx finish?
	call F_SYS_GetCount	
	cmp	r1,r2
	je	L_SPITimeout?
	r3=[P_SPI1_Misc]
	test r3,C_SPI1_RX_Full_Flag
	jz waitrx?		
	r2=8					//if tx finish, then read out RX FIFO data and store
cleRxFIFO?:
	r1=[P_SPI1_RXData]	
.if	Addressmode
	r3=[P_SPI1_RXData]	
	r3=r3 lsl 4
	r3=r3 lsl 4
	r3&=0xFF00
	r1|=r3
	ds:[r4++]=r1
	r2-=2	
.else	
	ds:[r4++]=r1
	r2-=1
.endif		
	jnz	cleRxFIFO?
	r5-=8
	jnz	L_TransLoop?	
endTX?:
	r1=0
	retf
	
L_SPIWriteShortFIFO?:
	r2=r5	
writeloop1?:	
	r3=0
	[P_SPI1_TXData]=r3		//send dummy byte
	r2-=1
	jnz	writeloop1?

txfinish?:
	call F_SYS_GetCount		
	r2=r1+3
waitrx1?:					//detect remain byte tx finish?
	call F_SYS_GetCount	
	cmp	r1,r2
	je	L_SPITimeout?
	r3=[P_SPI1_RXStatus]
	r3&=0x07
	cmp	r3,r5
	jne waitrx1?			
	r2=r5					//if tx finish, then read out RX FIFO data
cleRxFIFO1?:
	r1=[P_SPI1_RXData]	
.if	Addressmode
	r2-=1
	jz	skip?	
	r3=[P_SPI1_RXData]	
	r3=r3 lsl 4
	r3=r3 lsl 4
	r3&=0xFF00
	r1|=r3
	ds:[r4++]=r1
	r2-=1
	jnz	cleRxFIFO1?
	r1=0
	retf
skip?:	
	ds:[r4]=r1
	r1=0
	retf
.else	
	ds:[r4++]=r1
	r2-=1
	jnz	cleRxFIFO1?
	r1=0
	retf	
.endif

L_SPITimeout?:
	r1=-1
	retf
	


// ======================================================================================
// Function name: SPI1_TransmissionDMA
// Purpose		: SPI1 transmission by DMA. 
// Parameter	: int* R_SPI1_TXDataPoint: transmit data buffer Address. 
//				  int* R_SPI1_RXDataPoint: receive data buffer Address.
//				  long R_SPI1_DataLength: transmission data length
//				  int Mode: transmission mode. here, Mode=0,transmit address is constant and transmit address is byte mode, receive address is constant and recevie address is byte mode.   
//				    		  						 Mode=1,transmit address is increase and transmit address is byte mode, receive address is constant and recevie address is byte mode.	
//				    		  						 Mode=2,transmit address is constant and transmit address is word mode, receive address is constant and recevie address is byte mode.	
//				    		  						 Mode=3,transmit address is increase and transmit address is word mode, receive address is constant and recevie address is byte mode.	
//				    		  						 Mode=4,transmit address is constant and transmit address is byte mode, receive address is increase and recevie address is byte mode.	
//				    		  						 Mode=5,transmit address is increase and transmit address is byte mode, receive address is increase and recevie address is byte mode.	
//				    		  						 Mode=6,transmit address is constant and transmit address is word mode, receive address is increase and recevie address is byte mode.	
//				    		  						 Mode=7,transmit address is increase and transmit address is word mode, receive address is increase and recevie address is byte mode.	
//				    		  						 Mode=8,transmit address is constant and transmit address is byte mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=9,transmit address is increase and transmit address is byte mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=10,transmit address is constant and transmit address is word mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=11,transmit address is increase and transmit address is word mode, receive address is constant and recevie address is word mode.	
//				    		  						 Mode=12,transmit address is constant and transmit address is byte mode, receive address is increase and recevie address is word mode.	
//				    		  						 Mode=13,transmit address is increase and transmit address is byte mode, receive address is increase and recevie address is word mode.	
//				    		  						 Mode=14,transmit address is constant and transmit address is word mode, receive address is increase and recevie address is word mode.	
//				    		  						 Mode=15,transmit address is increase and transmit address is word mode, receive address is increase and recevie address is word mode.	
// Return		: r1=0: complete
//				  r1=-1: Timeout
// Destroy		: r1,r2 
// ======================================================================================
_SPI1_TransmissionDMA:
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]				
	[R_SPI1_TXDataPoint]=r1
	r1=[bp+4]				
	[R_SPI1_TXDataPoint+1]=r1
	r1=[bp+5]				
	[R_SPI1_RXDataPoint]=r1
	r1=[bp+6]				
	[R_SPI1_RXDataPoint+1]=r1
	r1=[bp+7]
	[R_SPI1_DataLength]=r1
	r1=[bp+8]
	[R_SPI1_DataLength+1]=r1
	r1=[bp+9]
	[R_SPI1_Mode]=r1
	call F_SPI1_TransmissionDMA
	pop bp from [sp]
	retf
	
F_SPI1_TransmissionDMA:
	r1=[P_DMA_INT]
	test r1,(C_DMA0_BusyFlag+C_DMA1_BusyFlag)		//wait DMA idle
	jnz	F_SPI1_TransmissionDMA	
	r1=0xef					//set SPI1 as DMA channel, channel0 is RX. channel1 is TX.
	[P_DMA_SS]=r1		
	r1=0x0200				//DMA reset
	[P_DMA_Ctrl0]=r1
	[P_DMA_Ctrl1]=r1
	
	r1=P_SPI1_RXData		//channel0(RX) register setup
	[P_DMA_SRC_AddrL0]=r1
	r1=0
	[P_DMA_SRC_AddrH0]=r1
	r1=[R_SPI1_RXDataPoint]
	[P_DMA_TAR_AddrL0]=r1
	r1=[R_SPI1_RXDataPoint+1]
	[P_DMA_TAR_AddrH0]=r1
	r1=[R_SPI1_DataLength]
	[P_DMA_TCountL0]=r1
	r1=[R_SPI1_DataLength+1]
	[P_DMA_TCountH0]=r1
	
	r1=[R_SPI1_TXDataPoint]		//channel1(TX) register setup
	[P_DMA_SRC_AddrL1]=r1
	r1=[R_SPI1_TXDataPoint+1]
	[P_DMA_SRC_AddrH1]=r1
	r1=P_SPI1_TXData
	[P_DMA_TAR_AddrL1]=r1
	r1=0
	[P_DMA_TAR_AddrH1]=r1
	r1=[R_SPI1_DataLength]
	[P_DMA_TCountL1]=r1
	r1=[R_SPI1_DataLength+1]
	[P_DMA_TCountH1]=r1	

	r1=0x03
	[P_DMA_INT]=r1			//clear int flag

	r1=[R_SPI1_Mode]		//get DMA control setup value according to mode selection 
	r1=r1 lsl 1
	r2=high6 T_DMAMode
	r1+=T_DMAMode
	r2+=0,carry
	ds=r2
	r2=ds:[r1++]	//DMA ch1 TX
	r3=ds:[r1++]	//DMA ch0 RX
	[P_DMA_Ctrl0]=r3
	[P_DMA_Ctrl1]=r2
	
	call F_SYS_GetCount	
	r2=r1+100
waitDMA?:			//wait DMA finish
	call F_SYS_GetCount	
	cmp	r1,r2
	je	L_timeout?
	r1=0xa005
	[P_WatchDog_Clear]=r1	
	r1=[P_DMA_INT]
	r1&=0x03
	cmp r1,0x03
	jne	waitDMA?
	[P_DMA_INT]=r1	//clear DMA int flag
	r1=0
	retf

L_timeout?:
	r1=-1
	retf




T_DMAMode:
//      ch1(TX)	ch0(RX)	
	.dw	0xf4cd, 0x78cd	// Mode=0,transmit address is constant and transmit address is byte mode, receive address is constant and recevie address is byte mode.   
	.dw	0xf44d, 0x78cd	// Mode=1,transmit address is increase and transmit address is byte mode, receive address is constant and recevie address is byte mode.	
	.dw	0xe4cd, 0x78cd	// Mode=2,transmit address is constant and transmit address is word mode, receive address is constant and recevie address is byte mode.	
	.dw 0xe44d, 0x78cd	// Mode=3,transmit address is increase and transmit address is word mode, receive address is constant and recevie address is byte mode.	
	.dw	0xf4cd, 0x788d	// Mode=4,transmit address is constant and transmit address is byte mode, receive address is increase and recevie address is byte mode.	
	.dw	0xf44d,	0x788d	// Mode=5,transmit address is increase and transmit address is byte mode, receive address is increase and recevie address is byte mode.	
	.dw	0xe4cd, 0x788d	// Mode=6,transmit address is constant and transmit address is word mode, receive address is increase and recevie address is byte mode.	
	.dw	0xe44d, 0x788d	// Mode=7,transmit address is increase and transmit address is word mode, receive address is increase and recevie address is byte mode.	
	.dw	0xf4cd,	0x58cd	// Mode=8,transmit address is constant and transmit address is byte mode, receive address is constant and recevie address is word mode.	
	.dw	0xf44d,	0x58cd	// Mode=9,transmit address is increase and transmit address is byte mode, receive address is constant and recevie address is word mode.	
	.dw	0xe4cd, 0x58cd	// Mode=10,transmit address is constant and transmit address is word mode, receive address is constant and recevie address is word mode.	
	.dw	0xe44d, 0x58cd	// Mode=11,transmit address is increase and transmit address is word mode, receive address is constant and recevie address is word mode.	
	.dw	0xf4cd, 0x588d	// Mode=12,transmit address is constant and transmit address is byte mode, receive address is increase and recevie address is word mode.	
	.dw	0xf44d, 0x588d	// Mode=13,transmit address is increase and transmit address is byte mode, receive address is increase and recevie address is word mode.	
	.dw 0xe4cd,	0x588d	// Mode=14,transmit address is constant and transmit address is word mode, receive address is increase and recevie address is word mode.	
	.dw	0xe44d, 0x588d	// Mode=15,transmit address is increase and transmit address is word mode, receive address is increase and recevie address is word mode.	

.endif	//.if MasterMode

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.if SlaveMode
//==========================================
// Public declare area
//==========================================
.public _Slave_SPI1_ReadOneByteData

.public _Slave_SPI1_Intial, F_Slave_SPI1_Intial
.public _Slave_SPI1_WriteOneByte, F_Slave_SPI1_WriteOneByte
.public _Slave_SPI1_WriteNByte, F_Slave_SPI1_WriteNByte
.public _Slave_SPI1_ReadNByte, F_Slave_SPI1_ReadNByte
.public _Slave_SPI1_Close, F_Slave_SPI1_Close
.public _Slave_SPI1_ISRService, F_Slave_SPI1_ISRService
.public _Slave_SPI1_Status, F_Slave_SPI1_Status

//==========================================
//Variable RAM declare area
//==========================================
Slave_SPI1_RAM:	.section	.ram
.var	R_Slave_SPI1_ReadOneByteData
.define _Slave_SPI1_ReadOneByteData		R_Slave_SPI1_ReadOneByteData	
R_Slave_SPI1_TXDataPoint: .dw 2 DUP(?)
.define	R_Slave_SPI1_RXDataPoint	R_Slave_SPI1_TXDataPoint	
R_Slave_SPI1_DataLength:	.dw	1 
R_Slave_SPI1_Status:		.dw	1		//0: no transmit
										//1: transmission one byte mode
										//2: write N byte mode
										//3: read N byte mode	

//==========================================
// code starting 
//==========================================
Slave_SPI1_DriverCode: .section	.code
// ======================================================================================
// Function name: Slave_SPI1_Intial
// Purpose		: SPI1 interface slave mode initial 
// Parameter	: None  
// Return		: None
// Destroy		: r1 
// ======================================================================================
_Slave_SPI1_Intial:	.proc
F_Slave_SPI1_Intial:
	r1 =0x0800			//reset
	[P_SPI1_Ctrl]=r1
	
	r1=[SPI1_CSPort+3]		//set SPI cs as input mode 
	r1&=~SPI1_CSpin
	[SPI1_CSPort+3]=r1	
	r1=[SPI1_CSPort+2]
	r1&=~SPI1_CSpin
	[SPI1_CSPort+2]=r1
	r1=[SPI1_CSPort]
	r1&=~SPI1_CSpin
	[SPI1_CSPort]=r1
	
	r1=[P_SPI1_Misc]		//enable smart mode
	r1|=C_SPI1_FIFO_Smart_Mode
	[P_SPI1_Misc]=r1

	r1=0x00					//disable TX int
	[P_SPI1_TXStatus]=r1
	[R_Slave_SPI1_Status]=r1
	r1=(C_SPI1_RX_INT_Flag+C_SPI1_RX_INT_EN+C_SPI1_RX_BUF_OVER_ERROR+C_SPI1_RX_FIFO_Level_1)	//set RX int and fifo level is 1
	[P_SPI1_RXStatus]=r1

	r1|=(C_SPI1_Enable+C_SPI1_SlaveMode+SPI1_CLKPhase+SPI1_CLKPolarity)
	[P_SPI1_Ctrl]=r1
	retf	
	.endp


// ======================================================================================
// Function name: Slave_SPI1_WriteOneByte
// Purpose		: SPI slave mode transmit and receive one byte data.               
// Parameter	: int TransmitData
// Return		: None.
//				  After this function call. User need call Slave_SPI1_Status to polling is master read data out?  
//                If Slave_SPI1_Status return 0, the read data will be put in Slave_SPI1_ReadOneByteData
// Destroy		: r1,r2 
// ======================================================================================	
_Slave_SPI1_WriteOneByte:
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]				//TX Data
	call F_Slave_SPI1_WriteOneByte
	pop bp from [sp]
	retf

F_Slave_SPI1_WriteOneByte:
	r2=[P_SPI1_Ctrl]	//reset SPI
	r2|=0x0800
	[P_SPI1_Ctrl]=r2
	
	[P_SPI1_TXData]=r1	

	r1=1
	[R_Slave_SPI1_DataLength]=r1	//one byte data to transmit
	[R_Slave_SPI1_Status]=r1		//1: transmission one byte mode

	r1=(C_SPI1_RX_INT_Flag+C_SPI1_RX_INT_EN+C_SPI1_RX_BUF_OVER_ERROR+C_SPI1_RX_FIFO_Level_1)	//set RX int and fifo level is 1
	[P_SPI1_RXStatus]=r1
    retf


// ======================================================================================
// Function name: Slave_SPI1_WriteNByte
// Purpose		: slave mode SPI N byte write(from buffer). here, the read data will be give up. 
// Parameter	: int* R_Slave_SPI1_TXDataPoint: TX data buffer Address, 
//				  int DataLength: transmit data length  
// Return		: None
//				  After this function call. User need call Slave_SPI1_Status to polling is master read data out?
//                If Slave_SPI1_Status return 0. Means transmit data is finish.  
// Destroy		: r1,r2,r3,r4,r5 
// ======================================================================================	
_Slave_SPI1_WriteNByte:
	push	bp to [sp]
	bp=sp+1
	r4=[bp+3]					//transmit data point low
	[R_Slave_SPI1_TXDataPoint]=r4
	r3=[bp+4]					//transmit data point high	
	[R_Slave_SPI1_TXDataPoint+1]=r3
	r5=[bp+5]					//data length
	[R_Slave_SPI1_DataLength]=r5
	call F_Slave_SPI1_WriteNByte
	pop bp from [sp]
	retf

F_Slave_SPI1_WriteNByte:
	r2=[P_SPI1_Ctrl]	//reset SPI
	r2|=0x0800
	[P_SPI1_Ctrl]=r2

	cmp	r5,0 					//if length 0 then end TX
	je	endTX?
	ds=r3						//data point ds
L_TransLoop?:
	cmp	r5,8
	jb	L_SPIWriteShortFIFO?
L_SPISequenceWrite?:			//length large than 8, then transmit 8 byte first and set RX FIFO level is 4byte.		
	r2=8
writeloop?:	
	r3=ds:[r4++]
	[P_SPI1_TXData]=r3	//low byte first
.if Addressmode
	r3=r3 lsr 4
	r3=r3 lsr 4
	[P_SPI1_TXData]=r3
	r2-=2	
.else
	r2-=1
.endif		
	jnz	writeloop?
	r3=ds
	[R_Slave_SPI1_TXDataPoint+1]=r3
	[R_Slave_SPI1_TXDataPoint]=r4
	r1=2
	[R_Slave_SPI1_Status]=r1	//2: write N byte mode
	r1=(C_SPI1_RX_INT_Flag+C_SPI1_RX_INT_EN+C_SPI1_RX_BUF_OVER_ERROR+C_SPI1_RX_FIFO_Level_4)	//set RX int and fifo level is 4 bytes
	[P_SPI1_RXStatus]=r1
	retf
	
L_SPIWriteShortFIFO?:
	r2=r5	
writeloop1?:	
	r3=ds:[r4++]
	[P_SPI1_TXData]=r3	//low byte first
.if Addressmode	
	r2-=1
	jz txfinish?		
	r3=r3 lsr 4
	r3=r3 lsr 4
	[P_SPI1_TXData]=r3
	r2-=1
.else
	r2-=1
.endif	
	jnz	writeloop1?

txfinish?:
	r1=2
	[R_Slave_SPI1_Status]=r1	//2: write N byte mode
	r1=(C_SPI1_RX_INT_Flag+C_SPI1_RX_INT_EN+C_SPI1_RX_BUF_OVER_ERROR)	
	cmp r5,0x04			//if data length large than 4, then set RX fifo level is 4 byte
	ja	intlevel4?		//if data length not large than 4, then set RX fifo level is length size
	r5-=1
	r5=r5 lsl 4
	r1|=r5
	jmp	skip?
intlevel4?:	
	r1|=C_SPI1_RX_FIFO_Level_4
skip?:	
	[P_SPI1_RXStatus]=r1
	retf

endTX?:			//no data need be transmitted.
	retf	

	
	
// ======================================================================================
// Function name: Slave_SPI1_ReadNByte
// Purpose		: Slave mode SPI N byte Read(to buffer). Here, SPI will send zero data out.  
// Parameter	: int* R_SPI1_RXDataPoint: receive data buffer Address.
//				  int R_SPI1_DataLength: receive data length 
// Return		: None
//				  After this function call. User need call Slave_SPI1_Status to polling is master send data out?  
//                If Slave_SPI1_Status return 0. Means receive data is finish.
// Destroy		: r1,r2,r3,r4,r5 
// ======================================================================================	
_Slave_SPI1_ReadNByte:
	push	bp to [sp]
	bp=sp+1
	r4=[bp+3]					//receive data point low 	
	[R_Slave_SPI1_RXDataPoint]=r4
	r3=[bp+4]					//receive data point high
	[R_Slave_SPI1_RXDataPoint+1]=r3
	r5=[bp+5]					//data length
	[R_Slave_SPI1_DataLength]=r5
	call F_Slave_SPI1_ReadNByte
	pop bp from [sp]
	retf


F_Slave_SPI1_ReadNByte:	
	r2=[P_SPI1_Ctrl]	//reset SPI
	r2|=0x0800
	[P_SPI1_Ctrl]=r2

	cmp	r5,0 					//if length 0 then end TX
	je	endRX?
L_TransLoop?:
	cmp	r5,8
	jb	L_SPIWriteShortFIFO?
L_SPISequenceWrite?:			//length large than 8, then transmit 8 byte first and set RX FIFO level is 4byte.
	r2=8
	r3=0
writeloop?:	
	[P_SPI1_TXData]=r3	//low byte first
	r2-=1
	jnz	writeloop?
	r1=3
	[R_Slave_SPI1_Status]=r1	//3: read N byte mode
	r1=(C_SPI1_RX_INT_Flag+C_SPI1_RX_INT_EN+C_SPI1_RX_BUF_OVER_ERROR+C_SPI1_RX_FIFO_Level_4)	//set RX int and fifo level is 4bytes
	[P_SPI1_RXStatus]=r1
	retf	

L_SPIWriteShortFIFO?:
	r2=r5	
	r3=0
writeloop1?:	
	[P_SPI1_TXData]=r3	//low byte first
	r2-=1
	jnz	writeloop1?

rxfinish?:
	r1=3
	[R_Slave_SPI1_Status]=r1	//2: write N byte mode
	r1=(C_SPI1_RX_INT_Flag+C_SPI1_RX_INT_EN+C_SPI1_RX_BUF_OVER_ERROR)	
	cmp r5,0x04			//if data length large than 4, then set RX fifo level is 4 byte
	ja	intlevel4?		//if data length not large than 4, then set RX fifo level is length size
	r5-=1
	r5=r5 lsl 4
	r1|=r5
	jmp	skip?
intlevel4?:	
	r1|=C_SPI1_RX_FIFO_Level_4
skip?:	
	[P_SPI1_RXStatus]=r1
	retf

endRX?:			//no data need be transmitted.	
	retf
	
	
// ======================================================================================
// Function name: Slave_SPI1_Close
// Purpose		: SPI1 function Disable.  
// Parameter	: None 
// Return		: None
// Destroy		: r1 
// ======================================================================================
_Slave_SPI1_Close:
F_Slave_SPI1_Close:
	r1=0x0800
	[P_SPI1_Ctrl]=r1
	retf	



// ======================================================================================
// Function name: Slave_SPI1_ISRService
// Purpose		: SPI1 slave mode interrupt service code. Put this function to SPI1 RX interrupt service routine   
// Parameter	: None 
// Return		: None
// Destroy		: r1,r2,r3,r4 
// ======================================================================================
_Slave_SPI1_ISRService:
F_Slave_SPI1_ISRService:
	r1=[P_SPI1_RXStatus]	//chech RX fifo is over run first, if over run, then stop receive int and set error flag
	test	r1,C_SPI1_RX_BUF_OVER_ERROR
	jz	conti?
	r2=8
clrloop?:	
	r1=[P_SPI1_RXData]
	r2-=1
	jnz	clrloop?
	r1=(C_SPI1_RX_INT_Flag+C_SPI1_RX_BUF_OVER_ERROR)
	[P_SPI1_RXStatus]=r1	
	r1=0xFFFF
	[R_Slave_SPI1_DataLength]=r1
	retf

conti?:
	r1=[R_Slave_SPI1_Status]
	cmp	r1,1
	je	OneByteMode
	cmp r1,2
	jne	skip?	
	goto	WriteNByte
skip?:	
	cmp	r1,3
	je	ReadNByte
	jmp	endservice
//--------------------------------------
OneByteMode:
	r1=[P_SPI1_RXData]
	[_Slave_SPI1_ReadOneByteData]=r1

endservice:
	r2=[P_SPI1_Ctrl]
	r2|=0x0800
	[P_SPI1_Ctrl]=r2
	r2=0
	[R_Slave_SPI1_Status]=r2
	[R_Slave_SPI1_DataLength]=r2
	r2=0x8000
	[P_SPI1_RXStatus]=r2	//clear adn disable SPI RX int
	retf

//------------------------------
ReadNByte:
	r3=[R_Slave_SPI1_RXDataPoint+1]
	ds=r3
	r4=[R_Slave_SPI1_RXDataPoint]
	r2=[R_Slave_SPI1_DataLength]
	r1=r2 lsr 2
	jnz	readconti?
readremain?:				//read out RX FIFO data
	r1=0
	[R_Slave_SPI1_DataLength]=r1
cleRxFIFO1?:
	r1=[P_SPI1_RXData]	
.if	Addressmode
	r2-=1
	jz	skip?	
	r3=[P_SPI1_RXData]	
	r3=r3 lsl 4
	r3=r3 lsl 4
	r3&=0xFF00
	r1|=r3
	ds:[r4++]=r1
	r2-=1
	jnz	cleRxFIFO1?
	goto	endservice	
skip?:	
	ds:[r4]=r1
.else
	ds:[r4++]=r1
	r2-=1
	jnz	cleRxFIFO1?
.endif		
	goto	endservice

readconti?:			//read 4 bytes
	r1=0
	[P_SPI1_TXData]=r1
	[P_SPI1_TXData]=r1
	[P_SPI1_TXData]=r1
	[P_SPI1_TXData]=r1
	r2=4
cleRxFIFO?:	
	r1=[P_SPI1_RXData]	
.if	Addressmode
	r3=[P_SPI1_RXData]	
	r3=r3 lsl 4
	r3=r3 lsl 4
	r3&=0xFF00
	r1|=r3
	ds:[r4++]=r1
	r2-=2
	jnz	cleRxFIFO?
.else
	ds:[r4++]=r1
	r2-=1
	jnz	cleRxFIFO?
.endif

rxfinish?:	
	r3=ds
	[R_Slave_SPI1_RXDataPoint+1]=r3
	[R_Slave_SPI1_RXDataPoint]=r4
	r2=[R_Slave_SPI1_DataLength]
	cmp	r2,4					//r2 is R_Slave_SPI1_DataLength
	ja	skip1?
	goto	endservice	
skip1?:
	r2-=4
	[R_Slave_SPI1_DataLength]=r2
	cmp	r2,4
	jbe	updateintlevel1?
	retf	
updateintlevel1?:
	r1=[P_SPI1_RXStatus]
	r1&=~0x00f0
	r2-=1
	r2=r2 lsl 4
	r1|=r2
	[P_SPI1_RXStatus]=r1	
	retf

//------------------------------
WriteNByte:
	r2=[R_Slave_SPI1_DataLength]
	r1=r2 lsr 2
	jnz	writeconti?
writeremain?:
	r1=0
	[R_Slave_SPI1_DataLength]=r1
	r3=[P_SPI1_RXData]
	r3=[P_SPI1_RXData]
	r3=[P_SPI1_RXData]
	r3=[P_SPI1_RXData]
	goto	endservice

writeconti?:		//send 4 bytes
	r1=4
skip?:
	r3=[P_SPI1_RXData]
	r3=[P_SPI1_RXData]
	r3=[P_SPI1_RXData]
	r3=[P_SPI1_RXData]
	r3=[R_Slave_SPI1_TXDataPoint+1]
	ds=r3
	r4=[R_Slave_SPI1_TXDataPoint]
writeloop?:	
	r3=ds:[r4++]
	[P_SPI1_TXData]=r3	//low byte first
.if Addressmode
	r1-=1
	jnz	txfinish?
	r3=r3 lsr 4
	r3=r3 lsr 4
	[P_SPI1_TXData]=r3
	r1-=1	
.else
	r1-=1
.endif		
	jnz	writeloop?
txfinish?:	
	r3=ds
	[R_Slave_SPI1_TXDataPoint+1]=r3
	[R_Slave_SPI1_TXDataPoint]=r4
	cmp	r2,4					//r2 is R_Slave_SPI1_DataLength
	ja	skip1?
	goto	endservice
skip1?:		
	r2-=4
	[R_Slave_SPI1_DataLength]=r2
	cmp	r2,4
	jbe	updateintlevel?
	retf	
updateintlevel?:
	r1=[P_SPI1_RXStatus]
	r1&=~0x00f0
	r2-=1
	r2=r2 lsl 4
	r1|=r2
	[P_SPI1_RXStatus]=r1	
	retf
//---------------------



// ======================================================================================
// Function name: Slave_SPI1_Status
// Purpose		: Slave mode data tramssion status check. After slave mode data transmit/receive function called. user need call this function to check is master had to teansmit/receive data.   
// Parameter	: None 
// Return		: remain data length.
//				  if return 0, means data transsiom is finish.
//				  if return 0xFFFF, means SPI slave receive over run. There are data mission.		
// Destroy		: r1 
// ======================================================================================
_Slave_SPI1_Status:
F_Slave_SPI1_Status:
	r1=[R_Slave_SPI1_DataLength]
	retf


.endif	//.if SlaveMode
	
