//==================================================================================
// The information contained herein is the exclusive property of
// Generalplus Technology Co. And shall not be distributed, reproduced,
// or disclosed in whole in part without prior written permission.
//       (C) COPYRIGHT 2009   Generalplus TECHNOLOGY CO.                            
//                   ALL RIGHTS RESERVED
// The entire notice above must be reproduced on all authorized copies.
//==================================================================================
//==================================================================================
// Name                  : SPI flash driver.asm
// Applied Body          : GPL951
// Programmer            : 
// Description           : SPI falsh driver code
// History version       : 
// Rev #      Date         Who			Comments
// -----	 -----------   ---------	--------------------------------------------
// 1.0	     2016/04/14   JackyLin		Oringinal Version
// 1.1       2016/12/20   JackyLin		Add 4 bytes addressing SPI Flash support.
// 1.2       2017/03/07   JackyLin		F_SPIFlash_WRSR function modify. Add WEL bit enable before WRSR command. 
//==================================================================================

//==========================================
// Include file area
//==========================================
.INCLUDE	GPL951_Body.inc
.include	..\SPIDriver\SPI0_driver.inc
.include	..\SPIDriver\SPI1_driver.inc

//==========================================
// Constant define area
//==========================================
//--------user define--------
.define	SPI1Used		//SPI0Used

.ifdef	SPI0Used			
.define	F_SPIFlash_Open				F_SPI0_Intial
.define	F_SPIFlash_CSHigh			F_SPI0_CSHigh
.define	F_SPIFlash_CSLow			F_SPI0_CSLow
.define	F_SPIFlash_WriteOneByte		F_SPI0_WriteOneByte
.define	F_SPIFlash_WriteNByte		F_SPI0_WriteNByte
.define	F_SPIFlash_ReadNByte		F_SPI0_ReadNByte
.define	R_SPI_ReadOneByteData		R_SPI0_ReadOneByteData
.else	
.ifdef	 SPI1Used
.define	F_SPIFlash_Open				F_SPI1_Intial
.define	F_SPIFlash_CSHigh			F_SPI1_CSHigh
.define	F_SPIFlash_CSLow			F_SPI1_CSLow
.define	F_SPIFlash_WriteOneByte		F_SPI1_WriteOneByte
.define	F_SPIFlash_WriteNByte		F_SPI1_WriteNByte
.define	F_SPIFlash_ReadNByte		F_SPI1_ReadNByte
.define	R_SPI_ReadOneByteData		R_SPI1_ReadOneByteData
.endif				
.endif	

//----------------------------------------------------
.define C_4k_Erase	1
.define C_64k_Erase	2
.define C_Chip_Erase	3

.define	C_SPIFlashReadCMD 			0x03
.define	C_SPIFlashSectorErase  		0x20  	//4k erase
.define	C_SPIFlashBlockErase    	0xD8  	//64k erase
.define	C_SPIFlashChipErase     	0x60
.define	C_SPIFlashPageProgram       0x02
.define	C_SPIFlashReadStatus        0x05  	//RDSR
.define	C_SPIFlashWriteStatus       0x01  	//WRSR
.define	C_SPIFlashWriteEnable       0x06  	//WREN
.define	C_SPIFlashWriteDisable      0x04  	//WRDI
.define	C_SPIFlashReadID            0x9F  	//RDID
.define	C_SPIFlashFastRead          0x0B
.define	C_SPIFlashPowerDown         0xB9
.define	C_SPIFlashReleasePowerDown  0xAB
.define	C_SPIFlashREMS              0x90  	//REMS / Read Electronic Manufacturer & Device ID

.define	C_SPIFlashReadCMD_4B        0x13
.define	C_SPIFlashSectorErase_4B    0x21  // 4k erase
.define	C_SPIFlashBlockErase_4B     0xDC  // 64k erase
.define	C_SPIFlashPageProgram_4B    0x12
//=============================================
// Flash Status Port Definition
//=============================================
.define	C_SPIFlashBusy      0x01
.define	C_SPIFlashWEL       0x02    	//Write Enable Latch
.define	C_SPIFlashBP0       0x04
.define	C_SPIFlashBP1       0x08
.define	C_SPIFlashBP2       0x10
.define	C_SPIFlashBP3       0x20    
.define	C_SPIFlashPEE       0x40    	//Program Erase Error
.define	C_SPIFlashSRWP      0x80   		//Status Register Write Protect


//==========================================
// External declare area
//==========================================
.external F_SYS_GetCount

//==========================================
// Public declare area
//==========================================
.PUBLIC F_SPIFlash_Initial
.PUBLIC F_SPIFlash_WREN
.PUBLIC F_SPIFlash_RDID
.PUBLIC F_SPIFlash_RDSR
.PUBLIC F_SPIFlash_WRSR
.PUBLIC F_SPIFlash_Read 
.PUBLIC F_SPIFlash_Erase 
.PUBLIC F_SPIFlash_Write
.PUBLIC F_SPIFlash_PowerDown
.PUBLIC F_SPIFlash_PowerDownRelease

.PUBLIC _SPIFlash_Initial
.PUBLIC _SPIFlash_WREN
.PUBLIC _SPIFlash_RDID
.PUBLIC _SPIFlash_RDSR
.PUBLIC _SPIFlash_WRSR
.PUBLIC _SPIFlash_Read 
.PUBLIC _SPIFlash_Erase 
.PUBLIC _SPIFlash_Write
.PUBLIC _SPIFlash_PowerDown
.PUBLIC _SPIFlash_PowerDownRelease

//==========================================
//Variable RAM declare area
//==========================================
SPIFlash_RAM:    .SECTION  .ram
R_SPIFlash_DataPtr:		.dw 2 dup(?) 
R_SPIFlash_Addr: 		.dw 2 dup(?)
.var	R_SPIFlash_DataLength
.var	R_SPIFlash_Temp
.var	R_SPIFlash_Type			//0: SPI flash 3byte address. 1: SPI Flash is 4byte address 

//==========================================
// code starting 
//==========================================
SPIFlash_Code: .section	.code

// ======================================================================================
// Function name: F_SPIFlash_Initial
// Purpose		: SPI Flash HW Initialize 
// Parameter	: int SPI_CLK_SEL. SPI clcok is system clock divided by 2/4/8/16/32/64/128
//				  Here, SPI_CLK_SEL=0 is system clock/2. SPI_CLK_SEL=1 is system clock/4. ... SPI_CLK_SEL=7 is system clock/128.   
// Return		: None
// Destroy		: r1,r2 
// ======================================================================================
_SPIFlash_Initial:
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]			//SPI_CLK_SEL
	call F_SPIFlash_Initial
	pop bp from [sp]
	retf

F_SPIFlash_Initial:
	call	F_SPIFlash_Open	
	call	F_SPIFlash_CSHigh
	r1=0
	[R_SPIFlash_Type]=r1		//default is 3 byte addressing
	retf
	
  
// ======================================================================================
// Function name : F_SPIFlash_WREN
// Purpose       : SPI Flash Write Enable
// Parameter     : None
// Return        : r1 = 0 , pass
//                 r1 = -1, timeout
// Destroy       : r1 
// ======================================================================================
F_SPIFlash_WREN:
_SPIFlash_WREN:
	call	F_SPIFlash_CSLow
	r1=C_SPIFlashWriteEnable      // 0x06
	call 	F_SPIFlash_WriteOneByte
	cmp r1,0
	jne	L_SPIFlashWRENTimeout?
	call	F_SPIFlash_CSHigh
	r1=0
	retf	

L_SPIFlashWRENTimeout?:
	call 	F_SPIFlash_CSHigh
	r1=-1
	retf	


// ======================================================================================
// Function name : F_SPIFlash_RDID
// Purpose       : Read SPI Flash ID
// Parameter     : char* R_SPIFlash_DataPtr: store ID data address
// Return        : R_SPIFlash_DataPtr[0] = manufactor ID
//                 R_SPIFlash_DataPtr[1] = memory type
//                 R_SPIFlash_DataPtr[2] = memory density
//                 r1=0 : pass , r1 = -1 timeout
// Destroy       : r1,r2,r3,r4   
// ======================================================================================
_SPIFlash_RDID:
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]			
	[R_SPIFlash_DataPtr]=r1
	r1=[bp+4]			
	[R_SPIFlash_DataPtr+1]=r1
	call F_SPIFlash_RDID
	pop bp from [sp]
	retf	

F_SPIFlash_RDID:
	call	F_SPIFlash_CSLow 
	r1=C_SPIFlashReadID           		//read ID command 0x9F
	call	F_SPIFlash_WriteOneByte
	cmp r1,0
	jne	F_SPIFlash_RDIDTimeOut?	
	r3=[R_SPIFlash_DataPtr+1]
	r4=[R_SPIFlash_DataPtr]
	r1=0	
	call F_SPIFlash_WriteOneByte		//read first byte
	cmp r1,0
	jne	F_SPIFlash_RDIDTimeOut?
	ds=r3	
	r1=[R_SPI_ReadOneByteData]			
	ds:[r4++]=r1						//store first byte				
	r1=0
	call F_SPIFlash_WriteOneByte		//read second byte	
	cmp r1,0
	jne	F_SPIFlash_RDIDTimeOut?
	r1=[R_SPI_ReadOneByteData]
	ds:[r4++]=r1						//store second byte
	r1=0
	call F_SPIFlash_WriteOneByte		//read third byte
	cmp r1,0
	jne	F_SPIFlash_RDIDTimeOut?
	r1=[R_SPI_ReadOneByteData]
	ds:[r4++]=r1						//store third byte
	call	F_SPIFlash_CSHigh
	
	r1=[R_SPI_ReadOneByteData]
	cmp r1,0x19					//256Mb	
	je	L_4ByteType?
	cmp r1,0x1A					//512Mb MX
	je	L_4ByteType?
	cmp r1,0x20					//512Mb Gigadevice
	je	L_4ByteType?
	cmp r1,0x1B					//1024Mb MX
	je	L_4ByteType?
	cmp r1,0x21					//1024Mb Gigadevice
	je	L_4ByteType?
L_below256Mb?:
	r1=0
	[R_SPIFlash_Type]=r1
	retf
L_4ByteType?:	
	r1=1
	[R_SPIFlash_Type]=r1
	r1=0
	retf	
	
F_SPIFlash_RDIDTimeOut?:
	call	F_SPIFlash_CSHigh	
	r1=-1
	retf	


  
// ======================================================================================
// Function name : F_SPIFlash_RDSR
// Purpose       : Read SPI Flash Status
// Parameter     : None
// Return        : if r1 = -1, timeout occured. 
//                 otherwise, r1 low byte return SPI Flash status.     
// Destroy       : r1, r2
// ======================================================================================
F_SPIFlash_RDSR:
_SPIFlash_RDSR:
	call	F_SPIFlash_CSLow 
	r1=C_SPIFlashReadStatus      		 //send read status command(0x05)		
	call	F_SPIFlash_WriteOneByte	
	cmp r1,0
	jne	L_GetSPIFlashRDSRTimeout?
	r1=0
	call	F_SPIFlash_WriteOneByte		//read back status data
	cmp r1,0
	jne	L_GetSPIFlashRDSRTimeout?
	call	F_SPIFlash_CSHigh
	r1=[R_SPI_ReadOneByteData]
	retf
	
L_GetSPIFlashRDSRTimeout?:
	call	F_SPIFlash_CSHigh
	r1=-1
	retf
	


// ======================================================================================
// Function name : F_SPIFlash_WRSR
// Purpose       : SPI Flash Write Status
// Parameter     : char StatusData
// Return        : r1 = 0 , pass
//                 r1 = -1, timeout
// Destroy       : r1,r2,r3
// ======================================================================================
_SPIFlash_WRSR:
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]				//status data
	call F_SPIFlash_WRSR
	pop bp from [sp]
	retf		


F_SPIFlash_WRSR:	
	r3=r1
 	call	F_SPIFlash_RDSR
	cmp r1,-1
	je	L_SPIFlashWRSRTimeouttmp?
	test r1,C_SPIFlashBusy
	jnz	 L_SPIFlash_WRSRBusy?
  	call	F_SPIFlash_WREN		
	cmp r1,0
 	jne	L_SPIFlashWRSRTimeouttmp?
  	call	F_SPIFlash_CSLow
   	r1=C_SPIFlashWriteStatus      	//send write status command(0x01)
  	call	F_SPIFlash_WriteOneByte
  	cmp r1,0
  	jne	L_SPIFlashWRSRTimeouttmp?
  	r1=r3
  	call	F_SPIFlash_WriteOneByte
  	cmp r1,0
  	jne	L_SPIFlashWRSRTimeouttmp?
  	call	F_SPIFlash_CSHigh
  	r1=0
  	retf

L_SPIFlashWRSRTimeouttmp?:
	call	F_SPIFlash_CSHigh  	
  	r1=-1
  	retf
  		
L_SPIFlash_WRSRBusy?:
	call	F_SPIFlash_CSHigh  	
  	r1=-2
  	retf 
   

// ======================================================================================
// Function name : F_SPIFlash_Read 
// Purpose       : SPI Flash read  
// Parameter     : long SPIFlash_Addr: SPI flash address
//                 int* StoreAddr: store data address   
//                 int DataLength: read data length
// Return        : r1 = 0 : complete
//                 r1 = -1: Time out, 
//                 r1 = -2: SPI Flash status is busy 
// Destroy       : r1, r2, r3, r4, r5
// ======================================================================================  
_SPIFlash_Read:
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]						//SPI address low
	[R_SPIFlash_Addr]=r1
	r1=[bp+4]						//SPI address high
	[R_SPIFlash_Addr+1]=r1
	r1=[bp+5]						//store data address
	[R_SPIFlash_DataPtr]=r1
	r1=[bp+6]
	[R_SPIFlash_DataPtr+1]=r1
	r1=[bp+7]						//data length
	[R_SPIFlash_DataLength]=r1
	call	F_SPIFlash_Read	
	pop bp from [sp]
	retf	


F_SPIFlash_Read:
	call	F_SPIFlash_RDSR
	cmp r1,-1
	je	L_SPIFlashReadTimeout?
	test r1,C_SPIFlashBusy
	jnz	 L_SPIFlash_ReadBusy?
	call	F_SPIFlash_CSLow
	r1=[R_SPIFlash_Type]
	jz	L_3ByteAddress?
	r1=C_SPIFlashReadCMD_4B				//first byte: read command(0x13)
	call	F_SPIFlash_WriteOneByte	
	cmp r1,0
	jne	L_SPIFlashReadTimeout?
	r1=[R_SPIFlash_Addr+1]
	r1=r1 lsr 4
	r1=r1 lsr 4
	call	F_SPIFlash_WriteOneByte		//second byte: address(32)
	cmp r1,0
	jne	L_SPIFlashReadTimeout?	
	jmp	L_Counti?
L_3ByteAddress?:
	r1=C_SPIFlashReadCMD           		//first byte: read command(0x03)
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_SPIFlashReadTimeout?	
L_Counti?:	
	r1=[R_SPIFlash_Addr+1]				//second byte: address(24)
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_SPIFlashReadTimeout?	
	r1=[R_SPIFlash_Addr]				//third byte: address(16)
	r1=r1 lsr 4
	r1=r1 lsr 4 
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_SPIFlashReadTimeout?	
	r1=[R_SPIFlash_Addr]				//fourth byte: address(8)
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_SPIFlashReadTimeout?	

	r4=[R_SPIFlash_DataPtr]					//receive data point low 	
	r3=[R_SPIFlash_DataPtr+1]				//receive data point high
	r5=[R_SPIFlash_DataLength]				//data length
	call F_SPIFlash_ReadNByte	
	cmp	r1,0
	jne	L_SPIFlashReadTimeout?
	call	F_SPIFlash_CSHigh
	r1=0
	retf	

L_SPIFlashReadTimeout?:             //return A=-1, Timeout
  	call	F_SPIFlash_CSHigh  
  	r1=-1
  	retf     
L_SPIFlash_ReadBusy?:
  	r1=-2	                        //SPI Flash status is busy
  	retf   


// ======================================================================================
// Function name : F_SPIFlash_Erase 
// Purpose       : SPI Flash Erase
// Parameter     : long SPIFlash_Addr: SPI flash address
//                 int EraseType: 1 = 4kB erase, 2 =64kB erase ,3= chip ease
// Return        : r1 = 0 : complete
//                 r1 = -1: SPI Time out, Timeout = 128Hz x10
//                 r1 = -2: SPI Flash status is busy      
//                 r1 = -3: EraseType ERROR 
// Destroy       : r1,r2
// ======================================================================================  
_SPIFlash_Erase:
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]						//SPI address low
	[R_SPIFlash_Addr]=r1
	r1=[bp+4]						//SPI address high
	[R_SPIFlash_Addr+1]=r1
	r3=[bp+5]						//erase type
	call	F_SPIFlash_Erase	
	pop bp from [sp]
	retf	


F_SPIFlash_Erase:
	call	F_SPIFlash_RDSR
	cmp r1,-1
	je	L_CheckSPIFlashEraseTimeouttmp?
	test r1,C_SPIFlashBusy
	jnz	 L_SPIFlash_EraseBusy?
	call	F_SPIFlash_WREN		
	cmp r1,0
  	jne	L_CheckSPIFlashEraseTimeouttmp?	
	call	F_SPIFlash_CSLow
	cmp	r3,1
	je	L_CheckSPIFlashSectorErase?
	cmp	r3,2
	je	L_CheckSPIFlashBlockErase?
	cmp	r3,3
	je	L_CheckSPIFlashChipErase?
	call	F_SPIFlash_CSHigh
	r1=-3
	retf

L_SPIFlash_EraseBusy?:
	r1=-2
	retf
	
L_CheckSPIFlashEraseTimeouttmp?:
	call	F_SPIFlash_CSHigh
	r1=-1
	retf	
	
L_CheckSPIFlashSectorErase?:
	r1=[R_SPIFlash_Type]
	jz	L_3ByteAddress?
	r1=C_SPIFlashSectorErase_4B
	jmp	L_WriteEraseCommond_4B?
L_3ByteAddress?:
	r1=C_SPIFlashSectorErase
	jmp	L_WriteEraseCommond?
	
L_CheckSPIFlashBlockErase?:
	r1=[R_SPIFlash_Type]
	jz	L_3ByteAddress1?
	r1=C_SPIFlashBlockErase_4B
	jmp	L_WriteEraseCommond_4B?
L_3ByteAddress1?:
	r1=C_SPIFlashBlockErase
	jmp	L_WriteEraseCommond?
	
L_CheckSPIFlashChipErase?:	
	r1=C_SPIFlashChipErase
	call	F_SPIFlash_WriteOneByte
	cmp r1,0
	jne	L_CheckSPIFlashEraseTimeouttmp?
	jmp	L_CheckSPIFlashEraseExit?
	
L_WriteEraseCommond_4B?:
	call	F_SPIFlash_WriteOneByte
	cmp r1,0
	jne	L_CheckSPIFlashEraseTimeouttmp?
	r1=[R_SPIFlash_Addr+1]
	r1=r1 lsr 4
	r1=r1 lsr 4	
L_WriteEraseCommond?:
	call	F_SPIFlash_WriteOneByte		//first byte erase command
  	cmp r1,0
  	jne	L_CheckSPIFlashEraseTimeouttmp?
	r1=[R_SPIFlash_Addr+1]				//second byte: address(24)
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_CheckSPIFlashEraseTimeouttmp?	
	r1=[R_SPIFlash_Addr]				//third byte: address(16)
	r1=r1 lsr 4
	r1=r1 lsr 4 
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_CheckSPIFlashEraseTimeouttmp?	
	r1=[R_SPIFlash_Addr]				//fourth byte: address(8)
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_CheckSPIFlashEraseTimeouttmp?	
L_CheckSPIFlashEraseExit?:	
	call	F_SPIFlash_CSHigh
	r1=0
	retf

	

// ======================================================================================
// Function name : F_SPIFlash_Write 
// Purpose       : SPI Flash Write 
// Parameter     : long SPIFlash_Addr: SPI flash address
//                 int* DataAddr: source data address  
//                 int DataLength: write data length
// Return        : r1 = 0 : Write complete, 
//                 r1 = -1: SPI Time out
//                 r1 = -2: SPI Flash status is busy      
// Destroy       : r1,r2,r3,r4,r5
// ======================================================================================
_SPIFlash_Write:
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]						//SPI address low
	[R_SPIFlash_Addr]=r1
	r1=[bp+4]						//SPI address high
	[R_SPIFlash_Addr+1]=r1
	r1=[bp+5]						//source data address low
	[R_SPIFlash_DataPtr]=r1
	r1=[bp+6]						//source data address high
	[R_SPIFlash_DataPtr+1]=r1
	r1=[bp+7]						//data length
	[R_SPIFlash_DataLength]=r1
	call	F_SPIFlash_Write
	pop bp from [sp]
	retf
	
L_SPIFlashWriteTimeoutTmp:
	r1=-1
	retf
L_SPIFlash_WriteBusy:
	r1=-2
	retf

F_SPIFlash_Write:
	call	F_SPIFlash_RDSR
	cmp r1,-1
	je	L_SPIFlashWriteTimeoutTmp
	test r1,C_SPIFlashBusy
	jnz	 L_SPIFlash_WriteBusy
L_SPIFlash_WriteStart?:	
	call	F_SPIFlash_WREN		
	cmp r1,0
  	jne	L_SPIFlashWriteTimeoutTmp		
	call	F_SPIFlash_CSLow
	//start write
	r1=[R_SPIFlash_Type]
	jz	L_3ByteAddress?
	r1=C_SPIFlashPageProgram_4B			//first byte: page program command
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
	jne	L_SPIFlashWriteTimeoutTmp	
	r1=[R_SPIFlash_Addr+1]	
	r1=r1 lsr 4
	r1=r1 lsr 4
	call	F_SPIFlash_WriteOneByte		//address(32)
  	cmp r1,0
  	jne	L_SPIFlashWriteTimeoutTmp
	jmp	L_Counti?
L_3ByteAddress?:
	r1=C_SPIFlashPageProgram           	//first byte: page program command
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_SPIFlashWriteTimeoutTmp	
L_Counti?:
	r1=[R_SPIFlash_Addr+1]				//second byte: address(24)
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_SPIFlashWriteTimeoutTmp	
	r1=[R_SPIFlash_Addr]				//third byte: address(16)
	r1=r1 lsr 4
	r1=r1 lsr 4 
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_SPIFlashWriteTimeoutTmp	
	r1=[R_SPIFlash_Addr]				//fourth byte: address(8)
	call	F_SPIFlash_WriteOneByte	
  	cmp r1,0
  	jne	L_SPIFlashWriteTimeoutTmp
	call	F_GetProgramPageRemainSize	//check, is over page program?, r1 is return remain size of page
	[R_SPIFlash_Temp]=r1
	r2=[R_SPIFlash_DataLength]
	cmp r2,r1		
	jae	L_SPIFlashOverPageFIFOWrite?
L_SPIFlashNoneOverPageFIFOWrite?:		//without over page	program
	r4=[R_SPIFlash_DataPtr]				//transmit data point low
	r3=[R_SPIFlash_DataPtr+1]			//transmit data point high	
	r5=[R_SPIFlash_DataLength]			//data length
	call	F_SPIFlash_WriteNByte
	cmp r1,0
	je	L_SPIFIFOWriteStartEnd?
	goto L_SPIFlashWriteTimeout?
	
L_SPIFlashOverPageFIFOWrite?:			//over page program
	r4=[R_SPIFlash_DataPtr]				//transmit data point low
	r3=[R_SPIFlash_DataPtr+1]			//transmit data point high	
	r5=r1								//data length
	call	F_SPIFlash_WriteNByte
	cmp r1,0
	jne	L_SPIFlashWriteTimeout?
	r1=[R_SPIFlash_Addr]				//update SPI Flash address
	r1+=[R_SPIFlash_Temp]
	[R_SPIFlash_Addr]=r1
	r1=[R_SPIFlash_Addr+1]
	r1+=0,carry
	[R_SPIFlash_Addr+1]=r1
	r1=[R_SPIFlash_DataPtr]				//update source data addr
	r2=[R_SPIFlash_Temp]
	r2=r2 lsr 1
	r1+=r2
	[R_SPIFlash_DataPtr]=r1
	r1=[R_SPIFlash_DataPtr+1]
	r1+=0,carry
	[R_SPIFlash_DataPtr+1]=r1

L_SPIFIFOWriteStartEnd?:
	call	F_SPIFlash_CSHigh         //CS high	 
L_GetSPIFlashWriteStatus?:
	call	F_SYS_GetCount
	r2=r1+10
L_SPIFlash_WriteTimeoutLoop1?:
	r1=0xa005
	[P_WatchDog_Clear]=r1
	call	F_SYS_GetCount
	cmp r2,r1
	je	L_SPIFlashWriteTimeout?
	call	F_SPIFlash_RDSR
	cmp	r1,-1
	je	L_SPIFlashWriteTimeout?
	test r1,C_SPIFlashBusy
	jnz	L_SPIFlash_WriteTimeoutLoop1?
	
L_CheckRemainDataLength?:
	r1=[R_SPIFlash_DataLength]
	r1-=[R_SPIFlash_Temp]
	[R_SPIFlash_DataLength]=r1
	jz	skip?	
	goto	L_SPIFlash_WriteStart?	
skip?:	
	r1=0
	retf

L_SPIFlashWriteTimeout?:
	r1=-1
	retf


// ======================================================================================
// Function name : F_GetProgramPageRemainSize    
// Purpose       : TranLegnth = 0x100 - R_SPIFlash_Addr[0];
// Parameter     : R_SPIFlash_Addr[0],
// Return        : r1: remain page size 
//                 
// Destroy       : r1,r2
// ======================================================================================    
F_GetProgramPageRemainSize:
	r1=0x100
	r2=[R_SPIFlash_Addr]
	r2&=0x0FF
	r1-=r2
	retf	

   

// ======================================================================================
// Function name : F_SPIFlash_PowerDown
// Purpose       : SPI Falsh deep power down
// Parameter     : none
// Return        : r1 = 0 : pass, ;         
//                 r1 = -1: SPI Time out
// Destroy       : r1,r2
// ======================================================================================    
_SPIFlash_PowerDown:
F_SPIFlash_PowerDown:
	call	F_SPIFlash_CSLow		
	r1=C_SPIFlashPowerDown
	call 	F_SPIFlash_WriteOneByte
	cmp r1,0
	jne	L_SPIFlashPowerDownTimeout?
	call	F_SPIFlash_CSHigh
	r1=0
	retf
	
L_SPIFlashPowerDownTimeout?:
	call	F_SPIFlash_CSHigh
	r1=-1
	retf	


// ======================================================================================
// Function name : F_SPIFlash_PowerDownRelease
// Purpose       : SPI Flash Deep power down release
// Parameter     : none
// Return        : r1 = 0 : pass, ;         
//                 r1 = -1: SPI Time out
// Destroy       : r1,r2
// ======================================================================================    
_SPIFlash_PowerDownRelease:
F_SPIFlash_PowerDownRelease:
	call	F_SPIFlash_CSLow		
	r1=C_SPIFlashReleasePowerDown
	call 	F_SPIFlash_WriteOneByte
	cmp r1,0
	jne	L_SPIFlashPowerDownReleaseTimeout?
	call	F_SPIFlash_CSHigh
	r1=0
	retf
	
L_SPIFlashPowerDownReleaseTimeout?:
	call	F_SPIFlash_CSHigh
	r1=-1
	retf	



