#ifndef	__SPIFLASHDRIVER_h__
#define	__SPIFLASHDRIVER_h__
//==================================================================================
// The information contained herein is the exclusive property of
// Generalplus Technology Co. And shall not be distributed, reproduced,
// or disclosed in whole in part without prior written permission.
//       (C) COPYRIGHT 2009   Generalplus TECHNOLOGY CO.                            
//                   ALL RIGHTS RESERVED
// The entire notice above must be reproduced on all authorized copies.
//==================================================================================
//==================================================================================
// Name                  : SPI Flash Driver.h
// Applied Body          : GPL951
// Programmer            : 
// Description           : SPI Flash driver header file
// History version       : 
// Rev #      Date         Who			Comments
// -----	 -----------   ---------	--------------------------------------------
// 1.0	     2016/04/14   JackyLin		Oringinal Version
//==================================================================================

//==========================================
// Include file area
//==========================================


//==========================================
// Constant define area
//==========================================
#define C_4k_Erase	1
#define C_64k_Erase	2
#define C_Chip_Erase	3

//==========================================
// External declare area
//==========================================

// ======================================================================================
// Function name: SPIFlash_Initial(int SPI_CLK_SEL)
// Purpose		: SPI Flash HW Initialize 
// Parameter	: int SPI_CLK_SEL. SPI clcok is system clock divided by 2/4/8/16/32/64/128 
//				  Here, SPI_CLK_SEL=0 is system clock/2. SPI_CLK_SEL=1 is system clock/4. ... SPI_CLK_SEL=7 is system clock/128.  	 
// Return		: None
// ======================================================================================
extern void SPIFlash_Initial(int SPI_CLK_SEL);


// ======================================================================================
// Function name : int SPIFlash_WREN(void)
// Purpose       : SPI Flash Write Enable
// Parameter     : None
// Return        : 0 , pass
//                 -1, timeout
// ======================================================================================
extern int SPIFlash_WREN(void);


// ======================================================================================
// Function name : int SPIFlash_RDID(unsigned char* R_SPIFlash_DataPtr); 
// Purpose       : Read SPI Flash ID
// Parameter     : char* R_SPIFlash_DataPtr: store ID data address
// Return        : R_SPIFlash_DataPtr[0] = manufactor ID
//                 R_SPIFlash_DataPtr[1] = memory type
//                 R_SPIFlash_DataPtr[2] = memory density
//                 r1=0 : pass , r1 = -1 timeout
// ======================================================================================
extern int SPIFlash_RDID(unsigned char* R_SPIFlash_DataPtr);


// ======================================================================================
// Function name : int SPIFlash_RDSR(void)
// Purpose       : Read SPI Flash Status
// Parameter     : None
// Return        : if return -1, timeout occured. 
//                 otherwise, low byte return SPI Flash status.     
// ======================================================================================
extern int SPIFlash_RDSR(void);


// ======================================================================================
// Function name : int SPIFlash_WRSR(char status)
// Purpose       : SPI Flash Write Status
// Parameter     : char StatusData
// Return        : 0 , pass
//                 -1, timeout
//				   -2, SPI Flash status is busy	
// ======================================================================================
extern int SPIFlash_WRSR(char status);


// ======================================================================================
// Function name : int F_SPIFlash_Read(long SPIFlash_Addr, int* StoreAddr, int dataLength)  
// Purpose       : SPI Flash read  
// Parameter     : long SPIFlash_Addr: SPI flash address
//                 int* StoreAddr: store data address   
//                 int DataLength: read data length
// Return        : 0 : complete
//                 -1: Time out, 
//                 -2: SPI Flash status is busy 
// ======================================================================================  
extern int SPIFlash_Read(long SPIFlash_Addr, int* StoreAddr, int dataLength);


// ======================================================================================
// Function name : int SPIFlash_Erase(long SPIFlash_Addr, int EraseType)  
// Purpose       : SPI Flash Erase
// Parameter     : long SPIFlash_Addr: SPI flash address
//                 int EraseType: 1 = 4kB erase, 2 =64kB erase ,3= chip ease
// Return        : 0 : complete
//                 -1: SPI Time out, Timeout = 128Hz x10
//                 -2: SPI Flash status is busy      
//                 -3: EraseType ERROR 
// ======================================================================================  
extern int SPIFlash_Erase(long SPIFlash_Addr, int EraseType);


// ======================================================================================
// Function name : int SPIFlash_Write(long SPIFlash_Addr, int* DataAddr, int dataLength) 
// Purpose       : SPI Flash Write 
// Parameter     : long SPIFlash_Addr: SPI flash address
//                 int* DataAddr: source data address  
//                 int DataLength: write data length
// Return        : 0 : Write complete, 
//                 -1: SPI Time out
//                 -2: SPI Flash status is busy      
// ======================================================================================
extern int SPIFlash_Write(long SPIFlash_Addr, int* DataAddr, int dataLength);


// ======================================================================================
// Function name : int SPIFlash_PowerDown(void)
// Purpose       : SPI Falsh deep power down
// Parameter     : none
// Return        : 0 : pass, ;         
//                 -1: SPI Time out
// ======================================================================================  
extern int SPIFlash_PowerDown(void);


// ======================================================================================
// Function name : int SPIFlash_PowerDownRelease(void)
// Purpose       : SPI Flash Deep power down release
// Parameter     : none
// Return        : 0 : pass, ;         
//                 1: SPI Time out
// ======================================================================================   
extern int SPIFlash_PowerDownRelease(void);


#endif
