//==================================================================================
// The information contained herein is the exclusive property of
// Generalplus Technology Co. And shall not be distributed, reproduced,
// or disclosed in whole in part without prior written permission.
//       (C) COPYRIGHT 2009   Generalplus TECHNOLOGY CO.                            
//                   ALL RIGHTS RESERVED
// The entire notice above must be reproduced on all authorized copies.
//==================================================================================
//==================================================================================
// Name                  : system.asm
// Applied Body          : GPL951
// Programmer            : 
// Description           : system code
// History version       : 
// Rev #      Date         Who			Comments
// -----	 -----------   ---------	--------------------------------------------

//==================================================================================
.INCLUDE	GPL951_body.inc

//==========================================
// Constant define area
//==========================================
.define	C_DebounceCnt	0x08
.define	C_SCAN_8_Bits	1		//1: 8 bit. 0: 16 bit

//==========================================
// Include file area
//==========================================


//==========================================
// External declare area
//==========================================


//==========================================
// Public declare area
//==========================================
.public _F_SYS_GotoHaltMode, F_SYS_GotoHaltMode
.public _F_SYS_GotoDeepHaltMode,F_SYS_GotoDeepHaltMode 
.public	_F_SYS_CounterEnable, F_SYS_CounterEnable
.public _F_SYS_CounterDisable, F_SYS_CounterDisable
.public _F_SYS_GetCount, F_SYS_GetCount
.public _F_SYS_Initial, F_SYS_Initial
.public _F_SYS_ServiceLoop, F_SYS_ServiceLoop
.public _F_SP_GetCh, F_SP_GetCh

.public R_128HzCounter, _R_128HzCounter

//==========================================
//Variable RAM declare area
//==========================================
System_RAM: .section .ram
.var	R_128HzCounter					//Record 128Hz INT counter value
.define	_R_128HzCounter	R_128HzCounter
.var	R_DebounceReg
.var	R_KeyBuf
.var	R_KeyStrobe
.var	R_DebounceCnt	

//==========================================
// code starting 
//==========================================
System_Code:	.section	.code
//=======================================================================================
// Function name: F_SYS_Initial
// Purpose		: System Initial
// Parameter	: 
// Return		: 
// Destroy		: 
//=======================================================================================
_F_SYS_Initial:	.proc
F_SYS_Initial:
	fir_mov off;
	
	call F_Key_Scan_Initial;
	call F_SYS_CounterEnable
	r1=0
	[R_128HzCounter]=r1
		
	retf
	.endp		



//****************************************************************
// Function    : F_Key_Scan_Initial
// Description : Key scan variable initialzation
// Destory     : R1
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
F_Key_Scan_Initial:	.proc
	R1 = 0x0000;
	[R_DebounceReg] = R1;
	[R_KeyBuf] = R1;
	[R_KeyStrobe] = R1;
	R1 = C_DebounceCnt;
	[R_DebounceCnt] = R1;
	retf;
	.endp


//****************************************************************
// Function    : F_SYS_ServiceLoop
// Description : Key scan and watchdog clear
// Destory     : R1, R2
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
_F_SYS_ServiceLoop: .proc
F_SYS_ServiceLoop:
	call F_Key_DebounceCnt_Down;		// debounce counter countdown
	call F_Key_Scan_ServiceLoop;		// key scan
	r1=C_WDT_Clear						// clear watchdog register
	[P_WatchDog_Clear]=r1
	retf;
	.endp;

//****************************************************************
// Function    : F_Key_DebounceCnt_Down
// Description : Debounce counter countdown
// Destory     : R1
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
F_Key_DebounceCnt_Down:
	R1 = [R_DebounceCnt];
	jz	L_DebounceCntZero;
	R1 -= 0x0001;
	[R_DebounceCnt] = R1;
L_DebounceCntZero:
	retf;
	
//****************************************************************
// Function    : F_Key_Scan_ServiceLoop
// Description : Get Key code from key pad(8 x 1 key pad)
// Destory     : R1, R2
// Parameter   : None
// Return      : None
// Note        : C_SCAN_8_Bits  = 1 , ScanKey 8bits
//               C_SCAN_8_Bits  = 0 , ScanKey 16bits  
//****************************************************************
F_Key_Scan_ServiceLoop:	.proc
	R1 = [P_IOB_Data];				// get key data from IOA  
.IF C_SCAN_8_Bits					// Notice, User can't operate the IOA in interrupt, because it will change the IOA configration.
	R1 &= 0x00FF; 			          
.ELSE	
	R1 &= 0xFFFF; 
.ENDIF

	R2 = [R_DebounceReg];
	[R_DebounceReg] = R1;
	cmp R2, [R_DebounceReg];
	je ?L_KS_StableTwoSample;

	R1 = C_DebounceCnt;				   // debounce counter reset
	[R_DebounceCnt] = R1;
	retf;

?L_KS_StableTwoSample:
	R1 = [R_DebounceCnt];
	jz ?L_KS_StableOverDebounce;
	retf;

?L_KS_StableOverDebounce:
	[R_DebounceCnt] = R1;
	R2 = [R_DebounceReg];
	R1 = [R_KeyBuf];
	[R_KeyBuf] = R2;

.IF  C_SCAN_8_Bits	
	R1 ^= 0x00FF;    					
	R1 = R1 and [R_KeyBuf];
	R1 &= 0x00FF;
.ELSE
	R1 ^= 0xFFFF;    					
	R1 = R1 and [R_KeyBuf];
	R1 &= 0xFFFF;
.ENDIF							
	
	R1 |= [R_KeyStrobe];
	[R_KeyStrobe] = R1;
	retf;
	.endp

//****************************************************************
// Function    : F_SP_GetCh
// Description : Get Keycode
// Destory     : R1, R2
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
_F_SP_GetCh:	.proc
F_SP_GetCh:
	R1 = [R_KeyStrobe];				// Get Key code
	R2 = 0x0000;					// Clear KeyStrobe for next key
	[R_KeyStrobe] = R2;
	retf;
	.endp




//=======================================================================================
// Function name: F_SYS_GotoHaltMode
// Purpose		: Go to Halt Mode
// Parameter	: 
// Return		: 
// Destroy		: 
//=======================================================================================
_F_SYS_GotoHaltMode:
F_SYS_GotoHaltMode:

	

	nop
	nop	
	nop
	
	

//=======================================================================================
// Function name: F_SYS_GotoDeepHaltMode
// Purpose		: Go to deep halt Mode
// Parameter	: NC
// Return		: 
// Destroy		: 
//=======================================================================================
_F_SYS_GotoDeepHaltMode:
F_SYS_GotoDeepHaltMode:

	
	NOP
	JMP		$
	nop
	nop
	
	
//=======================================================================================
// Function name: F_SYS_CounterEnable
// Purpose		: Enable 128Hz INT
// Parameter	: 
// Return		: 
// Destroy		: r1
//=======================================================================================
_F_SYS_CounterEnable:
F_SYS_CounterEnable:
	r1=(C_TimeBaseCFlag+C_TimeBaseCIntEn+C_TimeBaseCEn+C_TimeBaseC_128Hz)
	[P_TimeBaseC_Ctrl]=r1		
	irq on	
	retf
	
	
	
//=======================================================================================
// Function name: F_SYS_CounterDisable
// Purpose		: Disable 128Hz INT
// Parameter	: 
// Return		: None
// Destroy		: r1
//=======================================================================================
_F_SYS_CounterDisable:
F_SYS_CounterDisable:
	r1=(C_TimeBaseCFlag+C_TimeBaseCDis)
	[P_TimeBaseC_Ctrl]=r1
	retf
		
	
//=======================================================================================
// Function name: F_SYS_GetCount
// Purpose		: Get 128Hz Counter
// Parameter	: None
// Return		: R1 (Counter Value)
// Destroy		: R1
//=======================================================================================
_F_SYS_GetCount:
F_SYS_GetCount:
	R1=[R_128HzCounter]
	retf
	
.END


